# Map\[K: ([Hashable](collections-Hashable) val & [Equatable](builtin-Equatable)\[K val\] val), V: [Any](builtin-Any) val\]

A persistent map based on the Compressed Hash Array Mapped Prefix-tree from
'Optimizing Hash-Array Mapped Tries for Fast and Lean Immutable JVM
Collections' by Michael J. Steindorfer and Jurgen J. Vinju

## Usage
```
let empty: Map[String,U32] = Maps.empty[String,U32]() // {}
// Update returns a new map with the provided key set
// to the provided value. The old map is unchanged.
let m2 = m1("a") = 5 // {a: 5}
let m3 = m2("b") = 10 // {a: 5, b: 10}
let m4 = m3.remove("a") // {b: 10}
// You can create a new map from key value pairs.
let map = Maps.from[String,U32]([("a", 2), ("b", 3)]) // {a: 2, b: 3}
```


```pony
class val Map[K: (Hashable val & Equatable[K val] val), V: Any val]
```

## Constructors

### _empty

```pony
new val _empty()
: Map[K val, V val] val^
```

#### Returns

* [Map](collections-persistent-Map)\[K val, V val\] val^

---

### _create

```pony
new val _create(
  r: _MapNode[K val, V val] val,
  s: USize val)
: Map[K val, V val] val^
```
#### Parameters

*   r: [_MapNode](collections-persistent-_MapNode)\[K val, V val\] val
*   s: [USize](builtin-USize) val

#### Returns

* [Map](collections-persistent-Map)\[K val, V val\] val^

---

## Public Functions

### apply

Attempt to get the value corresponding to k.


```pony
fun val apply(
  k: K val)
: val->V val ?
```
#### Parameters

*   k: K val

#### Returns

* val->V val ?

---

### size

Return the amount of key-value pairs in the Map.


```pony
fun val size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### update

Update the value associated with the provided key.


```pony
fun val update(
  key: K val,
  value: V val)
: Map[K val, V val] val ?
```
#### Parameters

*   key: K val
*   value: V val

#### Returns

* [Map](collections-persistent-Map)\[K val, V val\] val ?

---

### remove

Try to remove the provided key from the Map.


```pony
fun val remove(
  k: K val)
: Map[K val, V val] val ?
```
#### Parameters

*   k: K val

#### Returns

* [Map](collections-persistent-Map)\[K val, V val\] val ?

---

### get_or_else

Get the value associated with provided key if present. Otherwise,
return the provided alternate value.


```pony
fun val get_or_else(
  k: K val,
  alt: val->V val)
: val->V val
```
#### Parameters

*   k: K val
*   alt: val->V val

#### Returns

* val->V val

---

### contains

Check whether the node contains the provided key.


```pony
fun val contains(
  k: K val)
: Bool val
```
#### Parameters

*   k: K val

#### Returns

* [Bool](builtin-Bool) val

---

### keys

```pony
fun val keys()
: MapKeys[K val, V val] ref
```

#### Returns

* [MapKeys](collections-persistent-MapKeys)\[K val, V val\] ref

---

### values

```pony
fun val values()
: MapValues[K val, V val] ref
```

#### Returns

* [MapValues](collections-persistent-MapValues)\[K val, V val\] ref

---

### pairs

```pony
fun val pairs()
: MapPairs[K val, V val] ref
```

#### Returns

* [MapPairs](collections-persistent-MapPairs)\[K val, V val\] ref

---

## Private Functions

### _root_node

```pony
fun box _root_node()
: _MapNode[K val, V val] val
```

#### Returns

* [_MapNode](collections-persistent-_MapNode)\[K val, V val\] val

---

