# Server

Runs an HTTP server. When routes are changed, the changes are only reflected
for new connections. Existing connections continue to use the old routes.


```pony
actor tag Server
```

## Constructors

### create

```pony
new tag create(
  auth: (AmbientAuth val | NetAuth val | TCPAuth val | 
    TCPListenAuth val),
  notify: ServerNotify iso,
  handler: RequestHandler val,
  logger: Logger val = reference,
  host: String val= "",
  service: String val= "0",
  limit: USize val = 0,
  sslctx: (SSLContext val | None val) = reference,
  reversedns: (AmbientAuth val | NetAuth val | DNSAuth val | 
    None val) = reference)
: Server tag^
```
#### Parameters

*   auth: ([AmbientAuth](builtin-AmbientAuth) val | [NetAuth](net-NetAuth) val | [TCPAuth](net-TCPAuth) val | 
    [TCPListenAuth](net-TCPListenAuth) val)
*   notify: [ServerNotify](net-http-ServerNotify) iso
*   handler: [RequestHandler](net-http-RequestHandler) val
*   logger: [Logger](net-http-Logger) val = reference
*   host: [String](builtin-String) val= ""
*   service: [String](builtin-String) val= "0"
*   limit: [USize](builtin-USize) val = 0
*   sslctx: ([SSLContext](net-ssl-SSLContext) val | [None](builtin-None) val) = reference
*   reversedns: ([AmbientAuth](builtin-AmbientAuth) val | [NetAuth](net-NetAuth) val | [DNSAuth](net-DNSAuth) val | 
    [None](builtin-None) val) = reference

#### Returns

* [Server](net-http-Server) tag^

---

## Public Behaviours

### set_handler

Replace the request handler.


```pony
be set_handler(
  handler: RequestHandler val)
```
#### Parameters

*   handler: [RequestHandler](net-http-RequestHandler) val

---

### set_logger

Replace the logger.


```pony
be set_logger(
  logger: Logger val)
```
#### Parameters

*   logger: [Logger](net-http-Logger) val

---

### dispose

Shut down the server.


```pony
be dispose()
```

---

## Public Functions

### local_address

Returns the locally bound address.


```pony
fun box local_address()
: IPAddress val
```

#### Returns

* [IPAddress](net-IPAddress) val

---

## Private Behaviours

### _listening

Called when we are listening.


```pony
be _listening(
  address: IPAddress val)
```
#### Parameters

*   address: [IPAddress](net-IPAddress) val

---

### _not_listening

Called when we fail to listen.


```pony
be _not_listening()
```

---

### _closed

Called when we stop listening.


```pony
be _closed()
```

---

