# _ClientConnection

Manages a persistent and possibly pipelined TCP connection to an HTTP server.


```pony
actor tag _ClientConnection
```

## Constructors

### create

```pony
new tag create(
  auth: (AmbientAuth val | NetAuth val | TCPAuth val | 
    TCPConnectAuth val),
  host: String val,
  service: String val,
  sslctx: (SSLContext val | None val) = reference,
  pipeline: Bool val = true)
: _ClientConnection tag^
```
#### Parameters

*   auth: ([AmbientAuth](builtin-AmbientAuth) val | [NetAuth](net-NetAuth) val | [TCPAuth](net-TCPAuth) val | 
    [TCPConnectAuth](net-TCPConnectAuth) val)
*   host: [String](builtin-String) val
*   service: [String](builtin-String) val
*   sslctx: ([SSLContext](net-ssl-SSLContext) val | [None](builtin-None) val) = reference
*   pipeline: [Bool](builtin-Bool) val = true

#### Returns

* [_ClientConnection](net-http-_ClientConnection) tag^

---

## Public Behaviours

### apply

Schedule a request.


```pony
be apply(
  request: Payload val)
```
#### Parameters

*   request: [Payload](net-http-Payload) val

---

### cancel

Cancel a request.


```pony
be cancel(
  request: Payload tag)
```
#### Parameters

*   request: [Payload](net-http-Payload) tag

---

## Private Behaviours

### _response

Call the request's handler and supply the response.


```pony
be _response(
  response: Payload iso)
```
#### Parameters

*   response: [Payload](net-http-Payload) iso

---

### _connected

The connection to the server has been established. Send pending requests.


```pony
be _connected(
  conn: TCPConnection tag)
```
#### Parameters

*   conn: [TCPConnection](net-TCPConnection) tag

---

### _connect_failed

The connection couldn't be established. Cancel all pending requests.


```pony
be _connect_failed(
  conn: TCPConnection tag)
```
#### Parameters

*   conn: [TCPConnection](net-TCPConnection) tag

---

### _auth_failed

The connection couldn't be authenticated. Cancel all pending requests.


```pony
be _auth_failed(
  conn: TCPConnection tag)
```
#### Parameters

*   conn: [TCPConnection](net-TCPConnection) tag

---

### _closed

The connection to the server has closed prematurely. Cancel everything.


```pony
be _closed(
  conn: TCPConnection tag)
```
#### Parameters

*   conn: [TCPConnection](net-TCPConnection) tag

---

## Private Functions

### _send

Send pending requests. If the connection is closed, open it. If we have
nothing to send and we aren't waiting on any responses, close the
connection.


```pony
fun ref _send()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _new_conn

Creates a new connection.


```pony
fun ref _new_conn()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _cancel_all

Cancel all pending requests.


```pony
fun ref _cancel_all()
: None val
```

#### Returns

* [None](builtin-None) val

---

