# _PayloadBuilder

This builds a Payload using received chunks of data.


```pony
class ref _PayloadBuilder
```

## Constructors

### request

```pony
new ref request()
: _PayloadBuilder ref^
```

#### Returns

* [_PayloadBuilder](net-http-_PayloadBuilder) ref^

---

### response

```pony
new ref response()
: _PayloadBuilder ref^
```

#### Returns

* [_PayloadBuilder](net-http-_PayloadBuilder) ref^

---

## Public Functions

### state

Returns the parse state.


```pony
fun box state()
: (_PayloadRequest val | _PayloadResponse val | _PayloadHeaders val | 
    _PayloadContentLength val | _PayloadChunkStart val | _PayloadChunk val | 
    _PayloadChunkEnd val | _PayloadBody val | _PayloadReady val | 
    _PayloadError val)
```

#### Returns

* ([_PayloadRequest](net-http-_PayloadRequest) val | [_PayloadResponse](net-http-_PayloadResponse) val | [_PayloadHeaders](net-http-_PayloadHeaders) val | 
    [_PayloadContentLength](net-http-_PayloadContentLength) val | [_PayloadChunkStart](net-http-_PayloadChunkStart) val | [_PayloadChunk](net-http-_PayloadChunk) val | 
    [_PayloadChunkEnd](net-http-_PayloadChunkEnd) val | [_PayloadBody](net-http-_PayloadBody) val | [_PayloadReady](net-http-_PayloadReady) val | 
    [_PayloadError](net-http-_PayloadError) val)

---

### parse

Parse available data based on our state. _ResponseBody is not listed here.
In that state, we wait for the connection to close and treat all pending
data as the response body.


```pony
fun ref parse(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

### done

Finish parsing. Returns the payload if it is ready, otherwise an empty
payload.


```pony
fun ref done()
: Payload iso^
```

#### Returns

* [Payload](net-http-Payload) iso^

---

### closed

The connection has closed, which may signal that all remaining data is the
payload body.


```pony
fun ref closed(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

## Private Functions

### _parse_request

Look for: "<Method> <URL> <Proto>".


```pony
fun ref _parse_request(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

### _parse_response

Look for: "<Proto> <Code> <Description>".


```pony
fun ref _parse_response(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

### _parse_headers

Look for: "<Key>:<Value>" or an empty line.


```pony
fun ref _parse_headers(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

### _parse_content_length

Look for _content_length available bytes.


```pony
fun ref _parse_content_length(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

### _parse_chunk_start

Look for the beginning of a chunk.


```pony
fun ref _parse_chunk_start(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

### _parse_chunk

Look for a chunk.


```pony
fun ref _parse_chunk(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

### _parse_chunk_end

Look for a blank line.


```pony
fun ref _parse_chunk_end(
  buffer: Reader ref)
: None val
```
#### Parameters

*   buffer: [Reader](buffered-Reader) ref

#### Returns

* [None](builtin-None) val

---

