# _ServerConnection

Manages a stream of requests to a server, ordering the responses.


```pony
actor tag _ServerConnection
```

## Constructors

### create

```pony
new tag create(
  handler: RequestHandler val,
  logger: Logger val,
  conn: TCPConnection tag,
  client_ip: String val)
: _ServerConnection tag^
```
#### Parameters

*   handler: [RequestHandler](net-http-RequestHandler) val
*   logger: [Logger](net-http-Logger) val
*   conn: [TCPConnection](net-TCPConnection) tag
*   client_ip: [String](builtin-String) val

#### Returns

* [_ServerConnection](net-http-_ServerConnection) tag^

---

## Public Behaviours

### dispatch

Dispatch requests. We can have many safe requests dispatched at once, or a
single unsafe request, since the client may expect an initial unsafe
request to alter the answer to following requests.


```pony
be dispatch(
  request: Payload iso)
```
#### Parameters

*   request: [Payload](net-http-Payload) iso

---

### answer

Associate a response with the given request and send responses in the
correct order.


```pony
be answer(
  request: Payload val,
  response: Payload val)
```
#### Parameters

*   request: [Payload](net-http-Payload) val
*   response: [Payload](net-http-Payload) val

---

## Public Functions

### is_safe

Return true for a safe request method, false otherwise.


```pony
fun box is_safe(
  method: String val)
: Bool val
```
#### Parameters

*   method: [String](builtin-String) val

#### Returns

* [Bool](builtin-Bool) val

---

## Private Functions

### _dispatch_pending

If we have pending requests, dispatch as many as we can.


```pony
fun ref _dispatch_pending()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _send_responses

Send available responses in the correct order.


```pony
fun ref _send_responses()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _send

Send a single response.


```pony
fun ref _send(
  request: Payload val,
  response: Payload val)
: None val
```
#### Parameters

*   request: [Payload](net-http-Payload) val
*   response: [Payload](net-http-Payload) val

#### Returns

* [None](builtin-None) val

---

