# Payload

An HTTP payload. For a response, the method indicates the status text. For a
request, the status is meaningless.


```pony
class iso Payload
```

## Constructors

### request

```pony
new iso request(
  method': String val= "GET",
  url': URL val = reference,
  handler': (ResponseHandler val | None val) = reference)
: Payload iso^
```
#### Parameters

*   method': [String](builtin-String) val= "GET"
*   url': [URL](net-http-URL) val = reference
*   handler': ([ResponseHandler](net-http-ResponseHandler) val | [None](builtin-None) val) = reference

#### Returns

* [Payload](net-http-Payload) iso^

---

### response

```pony
new iso response(
  status': Status val = reference,
  handler': (ResponseHandler val | None val) = reference)
: Payload iso^
```
#### Parameters

*   status': [Status](net-http-Status) val = reference
*   handler': ([ResponseHandler](net-http-ResponseHandler) val | [None](builtin-None) val) = reference

#### Returns

* [Payload](net-http-Payload) iso^

---

### _empty

```pony
new iso _empty(
  response': Bool val = true)
: Payload iso^
```
#### Parameters

*   response': [Bool](builtin-Bool) val = true

#### Returns

* [Payload](net-http-Payload) iso^

---

## Public fields

* var handler: ([ResponseHandler](net-http-ResponseHandler) val | [None](builtin-None) val)

---

* var proto: [String](builtin-String) val

---

* var status: [U16](builtin-U16) val

---

* var method: [String](builtin-String) val

---

* var url: [URL](net-http-URL) val

---

## Public Functions

### apply

Get a header.


```pony
fun box apply(
  key: String val)
: String val ?
```
#### Parameters

*   key: [String](builtin-String) val

#### Returns

* [String](builtin-String) val ?

---

### update

Set a header. If we've already received the header, append the value as a
comma separated list, as per RFC 2616 section 4.2.


```pony
fun ref update(
  key: String val,
  value: String val)
: Payload ref^
```
#### Parameters

*   key: [String](builtin-String) val
*   value: [String](builtin-String) val

#### Returns

* [Payload](net-http-Payload) ref^

---

### headers

Get the headers.


```pony
fun box headers()
: this->HashMap[String val, String val, HashEq[String val] val] ref
```

#### Returns

* this->[HashMap](collections-HashMap)\[[String](builtin-String) val, [String](builtin-String) val, [HashEq](collections-HashEq)\[[String](builtin-String) val\] val\] ref

---

### body

Get the body.


```pony
fun box body()
: this->Array[(String val | Array[U8 val] val)] ref
```

#### Returns

* this->[Array](builtin-Array)\[([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)\] ref

---

### body_size

Get the total size of the body.


```pony
fun box body_size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### add_chunk

Add a chunk to the body.


```pony
fun ref add_chunk(
  data: (String val | Array[U8 val] val))
: Payload ref^
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

#### Returns

* [Payload](net-http-Payload) ref^

---

### respond

Trigger the response handler.


```pony
fun iso respond(
  response': Payload iso)
: None val
```
#### Parameters

*   response': [Payload](net-http-Payload) iso

#### Returns

* [None](builtin-None) val

---

### fail

Trigger the response handler with an error payload.


```pony
fun iso fail()
: None val
```

#### Returns

* [None](builtin-None) val

---

## Private Functions

### _client_respond

Trigger the response handler. This is private to prevent request handlers
from responding to a request more than once.


```pony
fun val _client_respond(
  response': Payload iso)
: None val
```
#### Parameters

*   response': [Payload](net-http-Payload) iso

#### Returns

* [None](builtin-None) val

---

### _client_fail

Trigger the response handler with an error payload. This is private to
prevent request handlers from responding to a request more than once.


```pony
fun val _client_fail()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _write

Writes the payload to a TCP connection.


```pony
fun box _write(
  conn: TCPConnection tag,
  keepalive: Bool val = true)
: None val
```
#### Parameters

*   conn: [TCPConnection](net-TCPConnection) tag
*   keepalive: [Bool](builtin-Bool) val = true

#### Returns

* [None](builtin-None) val

---

### _write_request

Writes an an HTTP request.


```pony
fun box _write_request(
  conn: TCPConnection tag,
  keepalive: Bool val)
: None val
```
#### Parameters

*   conn: [TCPConnection](net-TCPConnection) tag
*   keepalive: [Bool](builtin-Bool) val

#### Returns

* [None](builtin-None) val

---

### _write_response

Write as an HTTP response.


```pony
fun box _write_response(
  conn: TCPConnection tag,
  keepalive: Bool val)
: None val
```
#### Parameters

*   conn: [TCPConnection](net-TCPConnection) tag
*   keepalive: [Bool](builtin-Bool) val

#### Returns

* [None](builtin-None) val

---

### _add_headers

Add the headers to the list.


```pony
fun box _add_headers(
  list: Array[(String val | Array[U8 val] val)] iso)
: Array[(String val | Array[U8 val] val)] iso^
```
#### Parameters

*   list: [Array](builtin-Array)\[([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)\] iso

#### Returns

* [Array](builtin-Array)\[([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)\] iso^

---

### _add_body

Add the body to the list.
TODO: don't include the body for HEAD, 204, 304 or 1xx


```pony
fun box _add_body(
  list: Array[(String val | Array[U8 val] val)] iso)
: Array[(String val | Array[U8 val] val)] iso^
```
#### Parameters

*   list: [Array](builtin-Array)\[([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)\] iso

#### Returns

* [Array](builtin-Array)\[([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)\] iso^

---

